/* Stuff to parse and compile text.
 */

/*

    Copyright (C) 1991-2003 The National Gallery

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */

/*

    These files are distributed with VIPS - http://www.vips.ecs.soton.ac.uk

 */

/* Maximum number of shared sections of code in a copy.
 */
#define MAX_RELOC (100)

#define TYPE_COMPILE (compile_get_type())
#define COMPILE( obj ) (GTK_CHECK_CAST( (obj), TYPE_COMPILE, Compile ))
#define COMPILE_CLASS( klass ) \
	(GTK_CHECK_CLASS_CAST( (klass), TYPE_COMPILE, CompileClass ))
#define IS_COMPILE( obj ) (GTK_CHECK_TYPE( (obj), TYPE_COMPILE ))
#define IS_COMPILE_CLASS( klass ) \
	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_COMPILE ))

/* What we track to parse and compile some text. 
 */
struct _Compile {
	GtkObject object;

	/* My instance vars.
	 */
	Symbol *sym;		/* We are part of this symbol, scopewise */

	GSList *exprs;		/* We are used by these expressions */

	gboolean is_klass;	/* True if this is a class */
	gboolean has_super;	/* True if has a super-class */

	char *text;		/* The original text */
	char *prhstext;		/* Parameters plus the RHS of the definition */
	char *rhstext;		/* Just the RHS of the definition */

	ParseNode *tree;	/* Parse tree we built */
	GSList *treefrag;	/* List of tree bits for easy freeing */

	SymTable *locals;	/* Table of names local to this def */
	int nparam;		/* Number of real parameters */
	GSList *param;		/* Pointers into locals for real params */
	int nsecret;		/* Number of secret parameters */
	GSList *secret;		/* Pointers into locals for secret params */
	Symbol *this;		/* If we are a class, the "this" local */
	Symbol *super;		/* If we are a class, the "super" local */
	GSList *children;	/* Symbols which we directly refer to */

	Element base; 		/* Base of compiled code */
	Heap *heap;		/* Heap containing compiled code */
};

typedef struct _CompileClass {
	GtkObjectClass parent_class;

	/* My methods.
	 */
} CompileClass;

Compile *compile_get_parent( Compile *compile );
void *compile_name_print( Compile *compile );

typedef void *(*map_compile_fn)( Compile *, void * );
Compile *compile_map_all( Compile *compile, map_compile_fn fn, void *a );

void compile_link_make( Compile *compile, Symbol *child );
void *compile_link_break( Compile *compile, Symbol *child );

GtkType compile_get_type( void );

void *compile_expr_link_break( Compile *compile, Expr *expr );
void *compile_expr_link_break_rev( Expr *expr, Compile *compile );
void compile_expr_link_make( Compile *compile, Expr *expr );

Compile *compile_new_root( Expr *expr );
Compile *compile_new_toplevel( Expr *expr );
Compile *compile_new_local( Expr *expr );

void *compile_heap( Compile *compile, gboolean optimise );

void compile_error_set( Compile *compile );
void compile_check( Compile *compile );
