/* Declarations for some basic util functions.
 */

/*

    Copyright (C) 1991-2003 The National Gallery

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */

/*

    These files are distributed with VIPS - http://www.vips.ecs.soton.ac.uk

*/

/* chartype strings. Useful with break_token().
 */
#define NUMERIC "0123456789"
#define WHITESPACE " \n\t"

/* No nulls! Handy for printf() %s
 */
#define NN( S ) ((S)?(S):"(null)")

/* Free things and set a string resource. FREEF frees a pointer type with a
 * specific free function, FREEFI frees an integer type (such as XtIntervalId)
 * with a free function.
 */

#define FREE( S ) { \
	if( S ) { \
		(void) im_free( (char *) (S) ); \
		(char *) (S) = NULL; \
	} \
}
#define FREEF( F, S ) { if( S ) { (void) F( S ); (S) = NULL; } }
#define FREEFI( F, S ) { if( S ) { (void) F( S ); (S) = 0; } }
#define SETSTR( S, V ) { \
	if( (S) != (V) ) { \
		if( !(S) || !(V) || strcmp( (S), (V) ) != 0 ) { \
			const char *sst = (V); \
			\
			FREE( S ); \
			if( sst ) \
				(S) = im_strdupn( sst ); \
		} \
	} \
}

#define FREEW( P ) { if( P ) { \
	gtk_widget_destroy( GTK_WIDGET( P ) ); (P) = NULL; } \
}
#define FREEFO( F, P ) { if( P ) { (F)( GTK_OBJECT( P ) ); (P) = NULL; } }
#define FREEO( P ) FREEFO( gtk_object_destroy, P )

#define FREESID( SID, OBJ ) { if( SID ) { \
	gtk_signal_disconnect( GTK_OBJECT( OBJ ), (SID) ); \
	(SID) = 0; } \
} \

extern const char *bandfmt_names[];
extern const int nbandfmt_names;

/* A string in the process of being written to ... multiple calls to 
 * buf_append add to it, on overflow append "..." and block further writes.
 */
typedef struct {
	char *base;		/* String base */
	int mx;			/* Maximum length */
	int i;			/* Current write point */
	gboolean full;		/* String has filled, block writes */
	int lasti;		/* For read-recent */
	gboolean dynamic;	/* We own the string with malloc() */
} BufInfo;

/* Init and append to one of the above.
 */
void buf_rewind( BufInfo *buf );
void buf_destroy( BufInfo *buf );
void buf_init( BufInfo *buf );
void buf_set_static( BufInfo *buf, char *base, int mx );
void buf_set_dynamic( BufInfo *buf, int mx );
void buf_init_static( BufInfo *buf, char *base, int mx );
void buf_init_dynamic( BufInfo *buf, int mx );
gboolean buf_appendns( BufInfo *buf, const char *str, int sz );
gboolean buf_appends( BufInfo *buf, const char *str );
gboolean buf_appendline( BufInfo *buf, const char *str );
gboolean buf_appendf( BufInfo *buf, const char *fmt, ... );
gboolean buf_vappendf( BufInfo *buf, const char *fmt, va_list ap );
gboolean buf_appendc( BufInfo *buf, char ch );
gboolean buf_appendsc( BufInfo *buf, const char *str );
gboolean buf_removec( BufInfo *buf, char ch );
gboolean buf_change( BufInfo *buf, const char *old, const char *new );
gboolean buf_isempty( BufInfo *buf );
gboolean buf_isfull( BufInfo *buf );
const char *buf_all( BufInfo *buf );
gboolean buf_appendd( BufInfo *buf, int d );
int buf_len( BufInfo *buf );

gboolean set_prop( xmlNode *xnode, const char *name, const char *fmt, ... );
gboolean set_sprop( xmlNode *xnode, const char *name, const char *value );
gboolean set_slprop( xmlNode *xnode, const char *name, GSList *labels );
gboolean set_dlprop( xmlNode *xnode, const char *name, double *values, int n );

gboolean get_sprop( xmlNode *xnode, const char *name, char *buf, int sz );
gboolean get_spropb( xmlNode *xnode, const char *name, BufInfo *buf );
gboolean get_iprop( xmlNode *xnode, const char *name, int *out );
gboolean get_dprop( xmlNode *xnode, const char *name, double *out );
gboolean get_bprop( xmlNode *xnode, const char *name, gboolean *out );
gboolean get_slprop( xmlNode *xnode, const char *name, GSList **out );
gboolean get_dlprop( xmlNode *xnode, const char *name, double **out );

void prettify_tree( xmlDoc *doc );

xmlNode *get_node( xmlNode *base, const char *name );

Rect *rect_dup( Rect *init );
void *rect_free( Rect *rect );

/* Like GFunc, but return a value.
 */
typedef gpointer (*SListMapFn)( gpointer, gpointer );
typedef gpointer (*SListMap2Fn)( gpointer, gpointer, gpointer );
typedef gpointer (*SListMap3Fn)( gpointer, gpointer, gpointer, gpointer );
typedef gpointer (*SListMap4Fn)( gpointer, gpointer, gpointer, gpointer,
	gpointer );
typedef gpointer (*SListMap5Fn)( gpointer, gpointer, gpointer, gpointer,
	gpointer, gpointer );
typedef gpointer (*SListFoldFn)( gpointer, gpointer, gpointer );
typedef gpointer (*SListFold2Fn)( gpointer, gpointer, gpointer, gpointer );

/* Like foreach, but allow abandon.
 */
void *slist_map( GSList *list, SListMapFn fn, gpointer a );
void *slist_map2( GSList *list, SListMap2Fn fn, gpointer a, gpointer b );
void *slist_map3( GSList *list, 
	SListMap3Fn fn, gpointer a, gpointer b, gpointer c );
void *slist_map4( GSList *list, 
	SListMap4Fn fn, gpointer a, gpointer b, gpointer c, gpointer d );
void *slist_map5( GSList *list, 
	SListMap5Fn fn, gpointer a, gpointer b, gpointer c, gpointer d,
	gpointer e );
void *slist_map_rev( GSList *list, SListMapFn fn, gpointer a );
void *slist_map2_rev( GSList *list, 
	SListMap2Fn fn, gpointer a, gpointer b );
void *slist_map3_rev( GSList *list, 
	SListMap3Fn fn, void *a, void *b, void *c );
void *map_equal( void *a, void *b );
gboolean slist_equal( GSList *l1, GSList *l2 );
void *slist_fold( GSList *list, void *start, SListFoldFn fn, void *a );
void *slist_fold2( GSList *list, 
	void *start, SListFold2Fn fn, void *a, void *b );
void slist_free_all( GSList *list );
GSList *slist_remove_all( GSList *list, gpointer data );

/* An slist, which tracks the end of the list, for fast append.
 */
typedef struct _Queue {
	GSList *list;
	GSList *tail;
} Queue;

Queue *queue_new( void );

/* All we need for now.
 */
void *queue_head( Queue *queue );
void queue_add( Queue *queue, void *data );
void queue_remove( Queue *q, void *data );

extern const char *error_string;	/* Set by procs to be helpful */

void error_block( void );		/* Block updates to error_string */
void error_unblock( void );
void error( const char *fmt, ... );
void ierrors( const char *fmt, ... );	/* Set error_string */
void verrors( const char *fmt, ... );	/* As above, but append VIPS message */

gboolean ispostfix( const char *a, const char *b );
gboolean isprefix( const char *a, const char *b );
gboolean iscaseprefix( const char *a, const char *b );
const char *findrightmost( const char *a, const char *b );
char *strcasestr( const char *haystack, const char *needle );
const char *skip_dir( const char *filename );
void change_suffix( const char *name, char *out, 
	const char *new, const char **olds, int nolds );

#ifndef HAVE_STRCCPY
char *strccpy( char *output, const char *input );
char *strecpy( char *output, const char *input );
#endif /*HAVE_STRCCPY*/
const char *my_strrspn( const char *p, const char *spn );

char *trim_nonalpha( char *text );
char *trim_white( char *text );

void *get_element( REGION *ireg, int x, int y, int b );

const char *decode_bandfmt( int f );
const char *decode_type( int t );
void get_image_info( BufInfo *buf, const char *name );

void expand_variables( const char *in, char *out );
void nativize_path( char *buf );

typedef void *(*call_string_fn)( const char *name, void *a, void *b, void *c );
void *call_string( call_string_fn fn, 
	const char *name, void *a, void *b, void *c );
void *call_stringva( call_string_fn fn, 
	const char *name, va_list ap, void *a, void *b, void *c );
void *call_stringf( call_string_fn fn, const char *name, ... );

char *im_strdupn( const char *str );
void putenvf( const char *name, ... );
gboolean existsf( const char *name, ... );
gboolean mkdirf( const char *name, ... );
int systemf( const char *fmt, ... );
FILE *popenf( const char *fmt, const char *mode, ... );
gboolean is_absolute( const char *fname );
gboolean is_valid_filename( const char *name );

/* Text IO to/from files. Track the filename too, to help error messages.
 */
typedef struct _iOpenFile {
	FILE *fp;	
	char *fname;		/* File we were passed to make this open_file */
	char *fname_real;	/* File we opened (maybe after search) */
	int last_errno;		/* On error, last value for errno */
	gboolean read;		/* True for open read, false for open write */
} iOpenFile;

void file_close( iOpenFile *of );
iOpenFile *file_open_read( const char *name, ... );
iOpenFile *file_open_read_stdin();
iOpenFile *file_open_write( const char *name, ... );
gboolean file_write( iOpenFile *of, const char *fmt, ... );
gboolean file_write_var( iOpenFile *of, const char *name, const char *value );
char *file_read( iOpenFile *of );

char *escape_percent( const char *in, char *out );
char *break_token( char *str, const char *brk );
const char *spc( int n );
void number_to_string( int n, char *buf );

unsigned int hash_name( const char *n );
unsigned int hash_pointer( void *p );

double find_space( const char *name );
void to_size( BufInfo *buf, double sz );
gboolean temp_name( char *name, const char *ext );
int findmaxmin( IMAGE *in, 
	int left, int top, int width, int height, double *min, double *max );

gboolean char_to_bool( char *str, void *out );
char *bool_to_char( gboolean b );

void increment_name( char *buf );

int extract_first_line( char *buf, char *str, int len );

void name_from_filename( const char *in, char *out );

void util_check_all_destroyed( void );
